import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;

/**
 * La classe IncassiSpese consente di memorizzare e analizzare una lista
 * di operazioni (incassi o spese, in base all'importo indicato).
 */
public class IncassiSpese {
    // Lista di operazioni effettuate
    private List<Operazione> operazioni;

    public IncassiSpese() {
        operazioni = new ArrayList<Operazione>();
    }

    // Metodo per l'aggiunta di una nuova operazione
    public void aggiuntaOperazione(double importo, String motivazione, LocalDate data) {
        operazioni.add(new Operazione(importo, motivazione, data));
    }

    // Metodo per la visualizzazione di tutte le operazioni effettuate
    public void visualizzazioneOperazioni() {
        // Consumer per la visualizzazione di una singola operazione
        Consumer<Operazione> visualizza = op -> {
            System.out.println("Data: " + op.getData());
            System.out.println("  Motivazione: " + op.getMotivazione());
            System.out.println("  Importo: € " + String.format("%.2f", op.getImporto()));
        };
        // Applicazione del consumer dichiarato sopra a ciascuna delle
        // operazioni memorizzate
        operazioni.forEach(visualizza);
    }

    // Metodo per la verifica della presenza di operazioni effettuate in
    // una precisa data (passata come parametro)
    public boolean controlloOperazioniInData(LocalDate data) {
        return operazioni.stream()
                    .anyMatch(op -> op.getData().equals(data));
    }

    // Metodo per il calcolo del totale delle spese effettuate
    // nell'intervallo di tempo (inizio,fine)
    public double calcoloSpeseInIntervallo(LocalDate inizio, LocalDate fine) {
        return operazioni.stream()
                .filter(op -> op.getImporto() < 0) // Restrizione alle sole operazioni di spesa
                .filter(op -> op.getData().isAfter(inizio)) // Eliminazione operazioni antecedenti alla data di inizio
                .filter(op -> op.getData().isBefore(fine)) // Eliminazione operazioni successive alla data di fine
                .map(op -> op.getImporto()) // Considerazione, per ciascuna operazione, del solo importo
                .reduce(0.0,(n,m) -> n + m); // Somma di tutti gli importi
    }

    // Metodo per il calcolo del totale delle spese determinate
    // dalla stessa motivazione
    public double calcoloSpesePerMotivazione(String motivazione) {
        return operazioni.stream()
                .filter(op -> op.getImporto() < 0) // Restrizione alle sole operazioni di spesa
                .filter(op -> op.getMotivazione().equalsIgnoreCase(motivazione)) // Eliminazione operazioni dovute a motivazioni diverse
                .map(op -> op.getImporto()) // Considerazione, per ciascuna operazione, del solo importo
                .reduce(0.0,(n,m) -> n + m); // Somma di tutti gli importi
    }

    // Metodo per il calcolo del totale degli incassi relativi
    // all'intervallo di tempo (inizio,fine)
    public double calcoloIncassiInIntervallo(LocalDate inizio, LocalDate fine) {
        return operazioni.stream()
                .filter(op -> op.getImporto() > 0) // Restrizione ai soli incassi
                .filter(op -> op.getData().isAfter(inizio)) // Eliminazione operazioni antecedenti alla data di inizio
                .filter(op -> op.getData().isBefore(fine)) // Eliminazione operazioni successive alla data di fine
                .map(op -> op.getImporto()) // Considerazione, per ciascuna operazione, del solo importo
                .reduce(0.0,(n,m) -> n + m); // Somma di tutti gli importi
    }

    // Metodo per il calcolo del totale degli incassi determinati
    // dalla stessa motivazione
    public double calcoloIncassiPerMotivazione(String motivazione) {
        return operazioni.stream()
                .filter(op -> op.getImporto() > 0) // Restrizione ai soli incassi
                .filter(op -> op.getMotivazione().equalsIgnoreCase(motivazione)) // Eliminazione operazioni dovute a motivazioni diverse
                .map(op -> op.getImporto()) // Considerazione, per ciascuna operazione, del solo importo
                .reduce(0.0,(n,m) -> n + m); // Somma di tutti gli importi
    }

    // Metodo per il calcolo del bilancio complessivo
    public double bilancioComplessivo() {
        return operazioni.stream()
                .map(op -> op.getImporto()) // Considerazione, per ciascuna operazione, del solo importo
                .reduce(0.0,(n,m) -> n + m); // Somma di tutti gli importi
    }

    // Metodo main eseguibile per il test della classe IncassiSpese
    public static void main(String[] args) {
        IncassiSpese opNegozio = new IncassiSpese();
        // Memorizzazione di alcune operazioni di esempio
        opNegozio.aggiuntaOperazione(-300.45, "acquisto prodotti ingrosso", LocalDate.parse("2017-11-14"));
        opNegozio.aggiuntaOperazione(-450.43, "tasse", LocalDate.parse("2017-12-31"));
        opNegozio.aggiuntaOperazione(-332.95, "Acquisto Prodotti Ingrosso", LocalDate.parse("2017-12-12"));
        opNegozio.aggiuntaOperazione(345.89, "Vendita Prodotti", LocalDate.parse("2017-12-14"));
        opNegozio.aggiuntaOperazione(875.89, "vendita prodotti", LocalDate.parse("2017-12-17"));
        opNegozio.aggiuntaOperazione(125.49, "rimborso tasse", LocalDate.parse("2017-03-31"));

        // Controllo esistenza operazioni in data 12-12-2017
        System.out.println("----------------------------------------------------");
        if(opNegozio.controlloOperazioniInData(LocalDate.parse("2017-12-12")))
            System.out.println("Ci sono operazioni effettuate il 12-Dic-17.");
        else
            System.out.println("Non ci sono operazioni effettuate il 12-Dic-17.");

        // Visualizzazione di tutte le operazioni effettuate
        System.out.println("----------------------------------------------------");
        opNegozio.visualizzazioneOperazioni();;

        // Calcolo delle spese effettuate nel mese di Dicembre 2017
        System.out.println("----------------------------------------------------");
        LocalDate inizioS = LocalDate.parse("2017-11-30");
        LocalDate fineS = LocalDate.parse("2018-01-01");
        System.out.println("Totale spese Dic-17: € " + String.format("%.2f", opNegozio.calcoloSpeseInIntervallo(inizioS,fineS)));

        // Calcolo delle spese dovute all'acquisto di prodotti
        System.out.println("----------------------------------------------------");
        System.out.println("Totale spese per acquisto prodotti: € " + String.format("%.2f", opNegozio.calcoloSpesePerMotivazione("acquisto prodotti ingrosso")));

        // Calcolo degli incassi del mese di Dicembre 2017
        System.out.println("----------------------------------------------------");
        LocalDate inizioI = LocalDate.parse("2017-11-30");
        LocalDate fineI = LocalDate.parse("2018-01-01");
        System.out.println("Totale incassi Dic-17: € " + String.format("%.2f", opNegozio.calcoloIncassiInIntervallo(inizioI,fineI)));

        // Calcolo degli incassi dovuti alla vendita di prodotti
        System.out.println("----------------------------------------------------");
        System.out.println("Totale incassi per vendita prodotti: € " + String.format("%.2f", opNegozio.calcoloIncassiPerMotivazione("vendita prodotti")));

        // Calcolo del bilancio complessivo
        System.out.println("----------------------------------------------------");
        System.out.println("Bilancio complessivo: € " + String.format("%.2f", opNegozio.bilancioComplessivo()));
    }

}
