package my.poligon;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Consumer;
import java.util.function.Function;

/**
 * Classe che consente la rappresentazione di un poligono
 * (data la lista dei suoi vertici)
 */
public class Poligono {

    // Lista di vertici del poligono
    private List<Punto> vertici;
    // Lista dei segmenti che compongono il poligono
    private List<Segmento> segmenti;


    public Poligono(List<Punto> vertici) {
        this.vertici = vertici;
        // Creazione della lista di segmenti a partire dai vertici
        this.segmenti = new ArrayList<Segmento>();
        for(int i=1; i<vertici.size(); i++)
            segmenti.add(new Segmento(vertici.get(i-1), vertici.get(i)));
    }

    // Metodo per la visualizzazione dei vertici che compongono il poligono
    public void visualizzazioneVertici() {
        Consumer<Punto> visualizzaPunto = p ->
                System.out.println("<" + p.getX() + "," + p.getY() + ">");
        vertici.forEach(visualizzaPunto);
    }

    // Calcolo del perimetro del poligono
    public double calcoloPerimetro() {
        Function<Segmento,Double> lunghezza = s -> {
            int xA = s.getA().getX();
            int xB = s.getB().getX();
            int yA = s.getA().getY();
            int yB = s.getB().getY();
            return Math.sqrt((xA-xB)*(xA-xB)+(yA-yB)*(yA-yB));
        };

        return segmenti.stream()
                .map(lunghezza)
                .reduce(0.0,(n1,n2) -> n1+n2);
    }

    // Determinazione dei quadranti del piano cartesiano toccati da (vertici del) poligono
    public Map<String,Boolean> determinazioneQuadrantiToccati() {
        Map<String,Boolean> quadrantiToccati = new HashMap<String,Boolean>();

        // Verifica sul primo quadrante
        if(vertici.stream().anyMatch(p -> p.getX()>0 && p.getY()>0))
            quadrantiToccati.put("I", true);
        else
            quadrantiToccati.put("I", false);

        // Verifica sul secondo quadrante
        if(vertici.stream().anyMatch(p -> p.getX()<0 && p.getY()>0))
            quadrantiToccati.put("II", true);
        else
            quadrantiToccati.put("II", false);

        // Verifica sul terzo quadrante
        if(vertici.stream().anyMatch(p -> p.getX()<0 && p.getY()<0))
            quadrantiToccati.put("III", true);
        else
            quadrantiToccati.put("III", false);

        // Verifica sul quarto quadrante
        if(vertici.stream().anyMatch(p -> p.getX()>0 && p.getY()<0))
            quadrantiToccati.put("IV", true);
        else
            quadrantiToccati.put("IV", false);

        return quadrantiToccati;
    }

    // Traslazione del poligono, dati i valori di spostamento lungo l'asse X
    // (deltaX) e lungo l'asse Y (deltaY)
    public void traslazione(int deltaX, int deltaY) {
        Consumer<Punto> traslazionePunto = p -> {
            p.setX(p.getX() + deltaX);
            p.setY(p.getY() + deltaY);
        };
        vertici.forEach(traslazionePunto);
    }

    // Metodo main (eseguibile) di test
    public static void main(String[] args) {
        // Creazione di un poligono
        List<Punto> vertici = new ArrayList<Punto>();
        vertici.add(new Punto(-3,4));
        vertici.add(new Punto(-3,-1));
        vertici.add(new Punto(1,3));
        Poligono pol = new Poligono(vertici);

        // Visualizzazione vertici
        System.out.println("Vertici del poligono:");
        pol.visualizzazioneVertici();
        System.out.println();

        // Calcolo del perimetro
        System.out.println("Perimetro del poligono: " + pol.calcoloPerimetro());
        System.out.println();

        // Quadranti toccati
        System.out.print("Quadranti toccati: ");
        Map<String,Boolean> quadrantiToccati = pol.determinazioneQuadrantiToccati();
        if(quadrantiToccati.get("I")) System.out.print("I ");
        if(quadrantiToccati.get("II")) System.out.print("II ");
        if(quadrantiToccati.get("III")) System.out.print("III ");
        if(quadrantiToccati.get("IV")) System.out.print("IV");
        System.out.println();

        // Traslazione vertici
        int deltaX = 3;
        int deltaY = 0;
        pol.traslazione(deltaX,deltaY);
        System.out.println("Vertici del poligono traslato (deltaX=" + deltaX + ",deltaY=" + deltaY + ":");
        pol.visualizzazioneVertici();
        System.out.println();

    }
}
